<?php
/**
 * Implementation of hook_cart_pane().
 */
function uc_upsell_cart_pane($items) {

  $panes[] = array(
    'id' => 'upsell_cart_form',
    'title' => t('Upsell Items &nbsp;'. l('(pane options)', 'admin/store/settings/upsell', array('attributes' => array('title' => 'configure the upsell pane')))),
    'enabled' => TRUE,
    'weight' => 1,
    'body' => uc_upsell_cart_pane_content($items),
  );
  return $panes;
}


/**
 * Get parts for cart pane
 */
function uc_upsell_cart_pane_content($data = '') {
  $related = uc_upsell_resolve_associates($data); // get related cart items (sets $msg too)
  return theme('pane_upsell', $related, $data);
}



/**
 * Theme function to render pane on cart page.
 */
function theme_pane_upsell($products, $msg = 'Related Products') {
  global $conf;
  if (empty($products)) return '';

  drupal_add_css(drupal_get_path('module', 'uc_upsell') .'/uc_upsell.css');
  $output = '<div id="cart-related-products"><p>'. t($msg) .'</p>';

  $output .= '<table id="related" width="100%"><tr><td align="center">';

  // Render a link and thumbnail for each product
  foreach ($products as $related) {
    $output .= '<div class="cart-related-item">';

    if ($related->field_image_cache && file_exists($related->image['filepath'] . $related->field_image_cache[0]['filepath'])) {
      $output .= l(theme('imagecache', uc_upsell_get_imagecache_preset(), $related->image['filepath'] . $related->field_image_cache[0]['filepath'], $related->title, $related->title, array('class' => 'buy-related-item')), drupal_get_path_alias("node/". $related->nid), array('attributes' => array('class' => 'buy-related-item'), 'html' => TRUE));
    } else {
      //use the default image
      $output .= l(theme('imagecache', uc_upsell_get_imagecache_preset(), 'files/default_product.jpg', $related->title, $related->title, array('class' => 'buy-related-item')), drupal_get_path_alias("node/". $related->nid), array('attributes' => array('class' => 'buy-related-item'), 'html' => TRUE));
    }

    // Linked title
    $output .= '<div class="upsell-block-product-title">';
    $output .= l($related->title, drupal_get_path_alias("node/". $related->nid), array('attributes' => array('class' => 'buy-related-item'), 'html' => TRUE));
    $output .= '</div>';

    // Only show an "add to cart" link if there is a value for Sell Price
    // TODO: Make destination configurable
    // TODO: Make this show an add to cart button instead?
    if ($related->sell_price) {
      $output .= l(t('Buy now ') . uc_currency_format($related->sell_price), "cart/add/p{$related->nid}_q1-i{$related->nid}Checkout", array('attributes' => array('class' => 'buy-related-item-buynow'), 'query' => 'destination=cart'));
    }

    $output .= '</div>';
  }

  $output .= '</td></tr></table>';
  $output .= '</div>';

  return $output;
}