<?php
/**
 * Get related items from the database and show them at the cart screen.
 * - If there aren't enough related products assigned in the admin, show a random selection.
 * - set's msg to the string to in the uc_upsell variable
 */
function uc_upsell_resolve_associates(&$msg = '') {
  $config = uc_upsell_get_config();
  $dpath = $_GET['q'];

  // If this is the cart...
  if ($dpath == 'cart') {
    $context = 'pane';
    $is_block = FALSE;
  }
  else {
    $is_block = TRUE;
    $node = node_load(arg(1));
    // If this is a product node...
    if (uc_product_is_product($node) == TRUE) {
      $product_nid = $node->nid;
      $context = 'product';
    }
    else {
      $context = 'node';
    }
  }
  //drupal_set_message("context: '$context' dpath: $dpath nid: {$matches[1]}");

  // If can't be enabled for this context, exit function
  if ($context != 'pane' && (($config['block']['enable'] == 1 && $context == 'product') || ($config['block']['enable'] == 0 && $context == 'node'))) {
    return array();
  }

  //drupal_set_message('Context' .$config['block'][$context]);

  // set vars to use when getting related items, based on the type of page being shown
  $vars = array(
    'max' => ($is_block) ? $config['block']['max'] : $config['pane']['max'],
    'use_random' => ($is_block) ? $config['block'][$context][(($context == 'node') ? 'source' : 'use_random')] : $config['pane']['use_random'],
  );

  // Get cart nids, as a base filter
  $cart_nids = array();
  $user_cart = uc_cart_get_contents();

  foreach ($user_cart as $cart) {
    $cart_nids[$cart->nid] = $cart->nid;
  }

  $related = array(); // List of related nodes to return

  $primary = 0; // Flag whether primary source is in result
  $sources = 0; // Flag

  // If source is a product node, begin with nodes related to this product that are not in cart or itself
  if (isset($product_nid)) {
    // Start list with nids related to the product node
    $related = uc_upsell_get_associate_nids(array(arg(1)), intval($vars['max']), $cart_nids);

    // If related is not empty...
    if (!empty($related)) {
      $primary = 1;
      $sources += 1;
    }
  }
  else {
    // We are on the cart, so we need to use $cart_nids as the source, as well as filter.
    $related = uc_upsell_get_associate_nids($cart_nids, $vars['max'], $cart_nids);
    if (!empty($related)) {
      $primary = 1;
      $sources += 1;
    }
  }

  //drupal_set_message('Related nodes found: '.count($related)); // Make sure everything is coming back A-OK.

  // If count of related nodes doesn't meet threshold and this is the pane or a block where we can use the cart...
  if (count($related) < $vars['max'] && ($context == 'pane' || ($context == 'node' && $config['block']['node']['source'] != 1) || $config['block']['product']['use_cart'])) {

    // If this is a node page block and cart items are required but missing, exit function
    if ($context == 'node' && $config['block']['req_cart'] && empty($cart_nids)) return array();

    // Get count of existing related nodes
    $rel_count = count($related);

    // Get/add nids related to the cart
    $related = array_merge($related, uc_upsell_get_associate_nids($cart_nids, $vars['max'] - count($related)));

    // if related count is greater than original...
    if (count($related) > $rel_count) {
      if (!isset($product_nid)) $primary = 1;
      $sources += 1;
    }
  }

  //drupal_set_message('number of related '.count($related).' vs max '.$vars['max']);
  // If count of related nodes doesn't meet threshold and can include random nodes, add them.
  if (count($related) < $vars['max'] && $vars['use_random']) {
    $random_nids = array();
    $filter_query = implode(",", array_merge($related, $cart_nids, array((isset($product_nid)) ? $product_nid : 0)));

    $randoms = db_query("SELECT `nid`
                FROM {node}
                WHERE `status` = 1
                AND `type` IN ('". implode("','", uc_product_product_types()) ."')
                AND NOT nid IN (". $filter_query .")
                ORDER BY RAND()
                LIMIT ". ($vars['max'] - count($related)));

    while ($random = db_fetch_array($randoms)) {
      $random_nids[] = $random['nid'];
    }

    if (!empty($random_nids)) {
      $sources += 1;
    }


    // Depending on our taxonomy settings, manipulate the array
    switch($config['global']['taxonomy']) {
      case 'ignore':
        $related = array_merge($related, $random_nids);
        break;
      case 'override':
        $node = node_load(arg(1));
        $related_by_term = uc_upsell_get_tids_for_intersect($terms, $node);
        $related = array_merge($related_by_term);
        break;
      case 'add':
        $node = node_load(arg(1));
        $related_by_term = uc_upsell_get_tids_for_intersect($terms, $node);
        $related = array_merge((array)$related, (array)$related_by_term);
        $related = array_merge($related, $random_nids);
        break;
    }
  }

  // If nothing is related, exit function
  if (empty($related)) return array();

  //dsm("primary: $primary, sources: $sources, strict msg? ". (($config['global']['loose_msg_req']) ? 'yes' : 'no'));

  // Set msg based on strict flag and number of sources...
  $msg = ($primary && ($sources == 1 || $config['global']['loose_msg_req'])) ? (($context == 'pane') ? $config['pane']['msg_related'] : $config['block'][$context]['msg_related']) : (($context == 'pane') ? $config['pane']['msg_mixed'] : $config['block'][$context]['msg_mixed']);

  // Shuffle the products
  if ($config['global']['shuffle']) shuffle($related);

  $related = array_slice($related, 0, $vars['max']);
  $products = array_map('node_load', $related);
  return $products;
}

/**
 * Find the associate nids
 */
function uc_upsell_get_associate_nids($source, $max = 0, $filter = array()) {
  //drupal_set_message('got source '.print_r($source, true) .' max '.$max .' filter '.print_r($filter, true));
  // Process args (a bit more robust here, since developers will likely use it)

  // Ensure source an array
  if (!is_array($source) && (is_numeric($source) || (is_object($source) && $source->nid))) $source = array((is_numeric($source)) ? $source : $source->nid);
  // Ensure filter an array
  if (!is_array($filter) && (is_numeric($filter) || (is_object($filter) && $filter->nid))) $filter = array((is_numeric($filter)) ? $filter : $filter->nid);
  // Ensure max is an integer
  if (!is_int($max) && is_numeric($max)) $max = abs(round($max));
  // If source or filter is not an array, or max is not an integer, exit function
  if (!is_array($source) || !is_array($filter) || !is_int($max)) {
    drupal_set_message('returning');
    return array();
  }

  $ary = array(); // Array to return
  $sought = array(); // Capture found nid

  foreach ($source as $nid) {
    if ((is_object($nid) && $nid->nid) || (is_array($nid) && $nid['nid'])) $nid = (is_object($nid)) ? $nid->nid : $nid['nid'];

    if (is_numeric($nid)) {
      $sought[] = $nid;
      $config = uc_upsell_get_config();
      $node = node_load($nid);
      $terms = taxonomy_node_get_terms($node);
      $related = db_result(db_query("SELECT `related_products` FROM {uc_upsell_products} WHERE nid = %d", $nid));
      $related_by_term = uc_upsell_get_tids_for_intersect($terms, $node);

      // If we found an array of related products specified in the node form, let's initialize them.
      if ($related && is_array(unserialize($related))) {
        $related = unserialize($related);
        $ary = array_merge($ary, $related);
      }
      else {
        $related = array();
      }

      // If we can get related nodes by taxonomy, let's do it.
      switch($config['global']['taxonomy']) {
        case 'ignore':
          //if (!is_array($related)) $related = unserialize($related);
          $ary = array_merge($ary, $related);
          break;

        case 'override':
          $ary = array_merge($ary, $related_by_term);
          break;

        case 'add':
          $ary = array_merge($ary, $related_by_term);
          break;

      }
    }
  }

  // Remove found nids with filter and already sought nids, make them unique
  $ary = uc_upsell_get_types_array($ary);
  $ary = uc_upsell_filter_types($ary);
  $ary = array_diff(array_keys($ary), $filter, $sought);
  $ary = array_unique($ary);

  // Shuffle related nodes
  shuffle($ary);

  // If max is valid, slice array to only show our configured maximum
  if ($max > 0) $ary = array_slice($ary, 0, $max);
  return $ary;
}


/**
 * Compute the taxonomy terms and store them as an array
 */
function uc_upsell_compute_terms() {
  $terms = db_query("SELECT a.name, b.tid, count( b.tid ) AS count
    FROM term_data a
    INNER JOIN term_node b ON a.tid = b.tid
    INNER JOIN node c ON b.nid = c.nid
    WHERE c.type IN ('". implode("','", uc_product_product_types()) ."')
    GROUP BY b.tid
    ORDER BY count DESC");

  $store = array();

  while ($term = db_fetch_object($terms)) {
    if ($term->count > 1) $store[] = intval($term->tid);
  }

  cache_set('uc_upsell_terms', serialize($store), 'cache', CACHE_TEMPORARY);
  //drupal_set_message('Terms were updated.');
}

/**
 * Get related nids based on term ids
 */
function uc_upsell_nids_by_term($intersect, $exclude_nid) {
  $related = db_query("select n.nid FROM node n
  INNER JOIN term_node t ON n.nid = t.nid
  WHERE t.tid IN ('". implode("','", $intersect) ."')
  AND n.type IN ('". implode("','", uc_product_product_types()) ."')
  AND n.nid <> %d
  GROUP BY n.title
  ORDER BY n.title", $exclude_nid);
  // Should exclude_nid be filtered out after the array is built? In case we need to exclude multiple nids?

  $nids = array();
  while ($result = db_fetch_object($related)) {
    $nids[] = $result->nid;
  }
  return $nids;
}

function uc_upsell_get_tids_for_intersect($terms, $node) {
  // Load the terms for the node in question
  $terms = taxonomy_node_get_terms($node);

  $node_tids = array();

  foreach ($terms AS $term) {
    //drupal_set_message('Term: '.$term->tid);
    $node_tids[] = $term->tid;
  }

  $allterms = cache_get('uc_upsell_terms');
  $intersect = array_intersect($node_tids, unserialize($allterms->data));
  $related_by_term = uc_upsell_nids_by_term($intersect, $node->nid);
  //drupal_set_message('related by term '.print_r($related_by_term, true));
  return $related_by_term;
}